/**********************************************************************
*  File:        HQ.C                                                  *
*  Purpose:     Host Buffer Queue Control Services                    *
*  Date:        13-Nov-1990                                           *
*  Author:      Howard D. Straus                                      *
*  Release: 1.00 - Original release                                   *
***********************************************************************
*                         COPYRIGHT NOTICE                            *
*=====================================================================*
*   Copyright 1990, 1991 by Howard D. Straus and Comtrol Corporation. *
*   All rights reserved.  Use or reproduction by any means of any     *
*   portion of this code contrary to the developers license agreement *
*   is expressly prohibited.                                          *
***********************************************************************/

#include "compiler.h"

#include <dos.h>
#ifdef  TURBO_C
#include <graphics.h>
#include <mem.h>
#else
#include <graph.h>
#include <memory.h>
#endif
#include <stdio.h>
#include <string.h>
#include <time.h>

#include "..\hostess\hostess.h"
#include "host.h"
#include "hostfns.h"
#include "hostext.h"


/**************************************************************************
*  name:        hq_alc                                                    *
*  purpose:     allocate a buffer from the free pool                      *
*  call:        bh = hq_alc();                                            *
*  parameters:  none.                                                     *
*  returns:                                                               *
*       BH_  *bh;   pointer to the allocated buffer                       *
*  method:      dequeues a buffer from the free pool, then clears it to   *
*       zero.                                                             *
***************************************************************************/

BH_     *
hq_alc()
{   BH_     *bh;                    /* buffer address */
        int     i;                      /* loop variable */

        if ((bh = hq_deq(&free_qcb)) != NULL)
        {   bh->bh_next = (BH_ *)NULL;
                bh->bh_txct = bh->bh_flgs = bh->bh_errs = 0;
                memset(bh, 0, sizeof(BH_));
        }
        return(bh);
}


/*************************************************************************
*  name:        hq_deq                                                   *
*  purpose:     dequeue a buffer from a queue                            *
*  call:        bh = hq_deq(qh);                                         *
*  parameter:                                                            *
*       QH_ *qh;  address of the queue (QC) from which the buffer is     *
*                 desired.                                               *
*  returns:                                                              *
*       BH_ *bh;  address of the buffer which was dequeued, or NULL      *
*                 if the queue was empty.                                *
*  method:      the queue is checked. If no buffers are on the queue,    *
*       a NULL pointer is returned.  Otherwise, the buffer at the head   *
*       of the queue is returned, and its forward pointer placed in the  *
*       QC's "first" pointer.  If the queue is now empty, the "last"     *
*       pointer is also set to point to the QC.                          *
**************************************************************************/

BH_        *
hq_deq(qh)
        QH_        *qh;             /* queue control block ptr */
{   BH_        *bh;             /* buffer pointer */
        int        s;               /* flags state save */

        DISABLE();                 /* disable interrupts */
        if ((bh = qh->qh_frst) == (BH_ *)qh) /* if no buf on queue */
                bh = NULL;              /* return queue empty indication */
        else
        {   qh->qh_frst = qh->qh_frst->bh_next; /* set first to next */
                if (qh->qh_frst == (BH_ *)qh) /* if queue now empty */
                        qh->qh_last = qh->qh_frst; /* adjust last ptr */
                bh->bh_next = NULL;     /* clear next buffer pointer */
        }
        ENABLE();                    /* restore interrupts */
        return(bh);                 /* return buffer to caller */
}


/**************************************************************************
*  name:        hq_enq                                                    *
*  purpose:     add a queue buffer to the end of a queue                  *
*  call:        hq_enq(bh, qh);                                           *
*  parameters:                                                            *
*       BH_ *bh;  pointer to the buffer to enqueue.                       *
*       QH_ *qh;  queue (QC) address.                                     *
*  returns: none.                                                         *
*  method:      hq_enq first sets the buffer "next" pointer to the QC     *
*   address, then sets the "next" pointer of the last buffer on the       *
*   queue and the queue "last" pointer to the new buffer's address.       *
***************************************************************************/

void
hq_enq(bh, qh)
        BH_         *bh;            /* buffer pointer */
        QH_         *qh;            /* queue control block */
{   int         s;              /* flags state save */

        if (bh == NULL)             /* check buffer pointer for validity */
                return;                 /* attempt to queue NULL buffer */
        if (qh == NULL)             /* check queue pointer for validity */
                return;                 /* attempt to queue to NULL queue */
        DISABLE();                 /* disable interrupts */
    bh->bh_next = (BH_ *)qh;    /* last buffer points to qhb */
        qh->qh_last->bh_next = bh;  /* add buffer to end of queue */
                                                                /* note: if this will be the first buffer */
                                                                /* on the queue, qh->qh_last->bh_next is */
                                                                /* really qh->qh_frst */
        qh->qh_last = bh;           /* last points to new buffer */
        ENABLE();                    /* restore interrupts */
}


/***********************************************************************
*  name:        hq_fre                                                 *
*  purpose:     free a buffer to the free buffer chain                 *
*  call:        hq_fre(bh);                                            *
*  parameter:                                                          *
*       BH_ *bh;  pointer to the buffer to free to the  heap.          *
*  returns: none.                                                      *
*  method:      returns the given buffer to the free buffer pool       *
************************************************************************/

void
hq_fre(bh)
        BH_         *bh;
{
        if (bh != (BH_ *)NULL)      /* if buffer given */
                hq_enq(bh, (QH_  *)&free_qcb); /* add it to free buffer pool */
}


/**********************************************************************
*  name:        hq_ini                                                *
*  purpose:     initialize the free buffer pool                       *
*  call:        hq_ini();                                             *
*  parameter:   none.                                                 *
*  returns:     none.                                                 *
*  method:      this routine frees buffer-sized blocks to the free    *
*       buffer queue.  All allocated buffers are freed.               *
***********************************************************************/

void
hq_ini()
{   BH_        *bh;            /* buffer pointer */
        int        i;              /* loop variable */

        hq_qci((QH_ *)&free_qcb);  /* initialize free pool */
        for (i = 0; i < N_BUFFERS; i++) /* by releasing all buffers */
                hq_fre(&mg_buffer[i]); /* to the free buffer pool */
}


/*************************************************************************
*  name:        hq_qci                                                   *
*  purpose:     initialize a buffer queue control block                  *
*  call:        hq_qci(qh);                                              *
*  parameter:                                                            *
*       QH_  *qh;  the address of the queue (QC) to initialize.          *
*  returns:     none.                                                    *
*  method:      both the first and last pointers in the queue are set    *
*       to the address of the queue (the parameter).  This is the indi-  *
*       cation that the queue is empty.                                  *
**************************************************************************/

void
hq_qci(qh)
        QH_          *qh;             /* queue control block address */
{   int           s;              /* flags state save */

        DISABLE();                   /* disable interrupts */
        qh->qh_frst = qh->qh_last = (BH_ *)qh; /* set up QCB pointers */
        ENABLE();                   /* restore interrupts */
}


/************************************************************************
*  name:        hq_qem                                                  *
*  purpose:     determine if a host queue is empty                      *
*  call:        empty = hq_qem(qh);                                     *
*  parameter:                                                           *
*       QH_  *qh;  the address of the queue (QC) to initialize.         *
*  returns:                                                             *
*       UINT empty;  if 0, queue is not empty                           *
*                    if non-zero, queue is empty                        *
*  method:      both the first and last pointers in the queue are set   *
*       to the address of the queue (the parameter).  This is the indi- *
*       cation that the queue is empty.                                 *
*************************************************************************/

UINT
hq_qem(qh)
        QH_         *qh;
{
        return(qh->qh_frst == (BH_ *)qh ? TRUE : FALSE);
}


                                /*  *** END OF FILE ***  */
