/**********************************************************************
*  File:        BQ.C                                                  *
*  Purpose:     COMTROL Hostess i Programmer's Toolkit Buffer Queue   *
*       Control Services                                              *
*  Date:        September 18, 1990                                    *
*  Author:      Howard D. Straus                                      *
*  Release: 1.00 - Original release                                   *
***********************************************************************
*                         COPYRIGHT NOTICE                            *
*=====================================================================*
*   Copyright 1990, 1991 by Howard D. Straus and Comtrol Corporation. *
*   All rights reserved.  Use or reproduction by any means of any     *
*   portion of this code contrary to the developers license agreement *
*   is expressly prohibited.                                          *
***********************************************************************/

#include    "compiler.h"
#include    "hostess.h"
#include    "externs.h"
#include    "proto.h"


/***********************************************************************
*  name:        bq_alc
*  purpose:     allocate a buffer from the free pool
*  call:        bp = bq_alc();
*  parameters:  none.
*  returns:
*       BF_ far *bp;  far pointer to the allocated buffer
*  method:      dequeues a buffer from the free pool, then clears it to
*       zero.
***********************************************************************/

BF_     far     *
bq_alc()
{   BF_     far     *bp;            /* buffer address */

        if ((bp = bq_deq(&free_pool)) != NULL)
        {   fmemset((void far *)bp, 0, sizeof(BF_)); /* clear buffer to 0 */
                --n_free_buffers;           /* decrement free buffer count */
        }
        return(bp);
}


/***********************************************************************
*  name:        bq_deq
*  purpose:     dequeue a buffer from a queue
*  call:        bp = bq_deq(qc);
*  parameter:
*       QC_ far *qc;  address of the queue (QC) from which the buffer is
*                   desired.
*  returns:
*       BF_ far  *bp;  address of the buffer which was dequeued, or NULL
*               if the queue was empty.
*  method:      the queue is checked. If no buffers are on the queue,
*       a NULL pointer is returned.  Otherwise, the buffer at the head
*       of the queue is returned, and its forward pointer placed in the
*       QC's "first" pointer.  If the queue is now empty, the "last"
*       pointer is also set to point to the QC.
**********************************************************************/

BF_    far      *
bq_deq(qc)
        QC_    far      *qc;        /* queue control block ptr */
{   BF_    far      *bp;        /* buffer pointer */
        int             s;          /* flags state save */

        s = dint();                 /* disable interrupts */
        if (bq_qem(qc))             /* if queue is empty */
                bp = NULL;              /* return queue empty indication */
        else
        {   bp = qc->qc_frst;       /* pick first off queue */
                qc->qc_frst = qc->qc_frst->bf_next; /* set first to next */
                if (qc->qc_frst == (BF_ far  *)qc) /* if queue now empty */
                        qc->qc_last = qc->qc_frst; /* adjust last ptr */
                bp->bf_next = NULL;     /* clear next buffer pointer */
        }
        eint(s);                    /* restore interrupts */
        return(bp);                 /* return buffer to caller */
}


/***********************************************************************
*  name:        bq_enq
*  purpose:     add a queue buffer to the end of a queue
*  call:        bq_enq(bp, qc);
*  parameters:
*       BF_ far *bp;  pointer to the buffer to enqueue.
*       QC_ far *qc;  queue (QC) address.
*  returns:     none.
*  method:      bq_enq first sets the buffer "next" pointer to the QC
*       address, then sets the "next" pointer of the last buffer on the
*       queue and the queue "last" pointer to the new buffer's address.
**********************************************************************/

void
bq_enq(bp, qc)
        BF_    far     *bp;         /* buffer pointer */
        QC_    far     *qc;         /* queue control block */
{   int             s;          /* flags state save */

        if (bp == NULL)             /* check buffer pointer for validity */
                return;         /* attempt to queue NULL buffer */
        if (qc == NULL)             /* check queue pointer for validity */
                return;                 /* attempt to queue to NULL queue */
        s = dint();                 /* disable interrupts */
        bp->bf_next = (BF_ far  *)qc; /* last buffer points to qcb */
        qc->qc_last->bf_next = bp;  /* add buffer to end of queue */
                                                                /* note: if this will be the first buffer */
                                                                /* on the queue, qc->qc_last->bf_next is */
                                                                /* really qc->qc_frst */
        qc->qc_last = bp;           /* last points to new buffer */
        eint(s);                    /* restore interrupts */
}


/***********************************************************************
*  name:        bq_fpq
*  purpose:     release all the buffers in a queue to the free pool
*  call:        bq_fpq(qc);
*  parameter:
*       QC_ far *qc;  the address of the queue to empty.
*  returns:     none.
*  method:      repetetively dequeues from the desired queue and frees
*       the buffers.
**********************************************************************/

void
bq_fpq(qc)
        QC_     far     *qc;            /* queue control block */
{   BF_     far     *bp;            /* primary thread */

        if (qc == NULL)                 /* if no queue */
                return;                     /* ignore call */

        while ((bp = bq_deq(qc)) != NULL) /* get the first buffer on queue */
                bq_fre(bp);                 /* free it */
        bq_qci(qc);                     /* reinitialize the queue */
}


/***********************************************************************
*  name:        bq_fre
*  purpose:     free a buffer to the free buffer chain
*  call:        bq_fre(bp);
*  parameter:
*       BF_ far  *bp;  pointer to the buffer to free to the far heap.
*  returns:     none.
*  method:      returns the given buffer to the free buffer pool
**********************************************************************/

void
bq_fre(bp)
        BF_     far     *bp;
{
        if (bp != (BF_ far  *)NULL)     /* if buffer given */
        {       bq_enq(bp, (QC_ far *)&free_pool);
                                        /* add it to free buffer pool */
                ++n_free_buffers;           /* increment free buffer count */
        }
}


/***********************************************************************
*  name:        bq_ini
*  purpose:     initialize the free buffer pool
*  call:        bq_ini();
*  parameter:   none.
*  returns:     none.
*  method:      repetitively allocates memory in buffer-sized blocks,
*       then releases them to the free pool.  All remaining memory is
*       given to the free pool.  This routine must be the last memory
*       requiring routine called as there will be no more memory after
*       bq_ini is done.
**********************************************************************/

void
bq_ini()
{   BF_     far     *bp;            /* buffer pointer */

        n_free_buffers = 0;                             /* init buffers in free pool */
        bq_qci((QC_ far *)&free_pool);  /* initialize free pool QCB */
        while ((bp = (BF_ far  *)memalloc(sizeof(BF_))) != NULL)
                bq_fre(bp);                 /* release block to free pool */
}


/***********************************************************************
*  name:        bq_qci
*  purpose:     initialize a buffer queue control block
*  call:        bq_qci(qc);
*  parameter:
*       QC_ far *qc;  the address of the queue (QC) to initialize.
*  returns:     none.
*  method:      both the first and last pointers in the queue are set
*       to the address of the queue (the parameter).  This is the indi-
*       cation that the queue is empty.
**********************************************************************/

void
bq_qci(qc)
        QC_     far     *qc;            /* queue control block address */
{   int             s;              /* flags state save */

        s = dint();                     /* disable interrupts */
        qc->qc_frst = qc->qc_last = (BF_ far  *)qc; /* set up QCB pointers */
        eint(s);                        /* restore interrupts */
}


/***********************************************************************
*  name:        bq_qem
*  purpose:     check for queue empty condition without dequeue
*  call:        r = bq_qem(qc);
*  parameter:
*       QC_ far *qc;  the address of the queue (QC) to initialize.
*  returns:
*       int r;  if non-zero, the queue is empty; if 0, the queue is
*               not empty
*  method:      if the pointer to the first on the queue points to
*       the queue control block itself, the queue is empty.
**********************************************************************/

int
bq_qem(qc)
        QC_     far     *qc;            /* queue control block */
{
        if (qc->qc_frst == (BF_ far  *)qc) /* if nothing on queue */
                return(TRUE);
        return(FALSE);
}


                                        /*  *** END OF FILE ***  */
